==> loncapa-2.7.1.tar.gz: loncapa-2.7.1/loncom/xml/lontexconvert.pm <==
# The LearningOnline Network with CAPA
# TeX Conversion Module
#
# $Id: lontexconvert.pm,v 1.89 2008/09/10 19:45:23 droeschl Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# Copyright for TtHfunc and TtMfunc by Ian Hutchinson. 
# TtHfunc and TtMfunc (the "Code") may be compiled and linked into 
# binary executable programs or libraries distributed by the 
# Michigan State University (the "Licensee"), but any binaries so 
# distributed are hereby licensed only for use in the context
# of a program or computational system for which the Licensee is the 
# primary author or distributor, and which performs substantial 
# additional tasks beyond the translation of (La)TeX into HTML.
# The C source of the Code may not be distributed by the Licensee
# to any other parties under any circumstances.
#

package Apache::lontexconvert;

use strict;
use tth();
use vars qw($errorstring);
#use Apache::lonxml();
use Apache::lonlocal;
use Apache::lonnet;
use lib '/home/httpd/lib/perl/';
use LONCAPA;
 

# ====================================================================== Header

sub init_tth {
    my $options=$env{'course.'.$env{'request.course.id'}.'.tthoptions'};
    if ($options =~ /\S/) {
	$options = ' '.$options;
    } else {
	undef($options);
    }
    if ($env{'browser.mathml'}) {
	&tth::ttminit();
	if ($env{'browser.unicode'}) {
	    &tth::ttmoptions('-L -u1'.$options);
	} else {
	    &tth::ttmoptions('-L -u0'.$options);
	}
    } else {
	&tth::tthinit();
	if ($env{'browser.unicode'}) {
	    &tth::tthoptions('-L -u1'.$options);
	} else {
	    &tth::tthoptions('-L -u0'.$options);
	}
    }
}

# ================================================================== Conversion

$Apache::lontexconvert::messedup=0;

# we need this routine because &converted can get called from inside
# of the safespace (through &xmlparse('<m>stuff</m>') which doesn't
# allow the opcode for alarm, so we need to compile this before we get
# into the safe space since opcode checks only occur at compile time
sub convert_real {
    my ($texstring)=@_;
    my ($xmlstring,$errorstring);
    local $SIG{SEGV}=sub { $Apache::lontexconvert::messedup=1; die; };
    local $SIG{ALRM}=sub { 
	&Apache::lonnet::logthis("ALRM");
	$xmlstring='['.&mt("TeX unconverted due to errors").']';
	$Apache::lontexconvert::messedup=1;
	die &mt("TeX unconverted due to errors");
    };
    &Apache::lonxml::start_alarm();
    if ($env{'browser.mathml'}) {
	$xmlstring=&tth::ttm($$texstring);
	$xmlstring=~s/\<math\>/\<math xmlns=\"\&mathns\;\"\>/g;
	$xmlstring=~s/\<br\>/\<br\/\>/g;
	$xmlstring=~s/\<p\>/\<p\>\<\/p\>/g;
	$errorstring.=&tth::ttmerror();
    } else {
	$xmlstring=&tth::tth($$texstring);
	$errorstring.=&tth::ttherror();
	$xmlstring=~s-</font(\s*)>-</font>-g;
    }
    $xmlstring=~s/^\s*\<br clear\=\"all\"/\<br/s;
    $xmlstring=~s/^\s*//;
    $xmlstring=~s/\s*$//;
    #
    # \rightleftharpoons is not converted by tth but maps
    # reasonably well to &#8660;.  If we get many more of these,
    # we're going to need to have a translation sub.
    #
    my $lrharpoon = pack("U", 0x21cc);
    $xmlstring=~s/\\rightleftharpoons/$lrharpoon/g;

    &Apache::lonxml::end_alarm();
    return ($xmlstring,$errorstring);
}

sub tth_converted {
    my $texstring=shift;
    my $xmlstring='['.&mt('UNDISPLAYABLE').']';
    if ($Apache::lontexconvert::messedup) {
	return '['.&mt('TeX unconverted due to previous errors').']';
    }
    $$texstring ='\\documentstyle{article}'.$$texstring;

    eval(<<'ENDCONV');
    ($xmlstring,$errorstring)=&convert_real($texstring)
ENDCONV
    if ($@) {
	$errorstring.=&mt("Evaluation Error: ").$@;
	$Apache::lontexconvert::messedup=1;
    }
    if ($Apache::lontexconvert::messedup || &tth::tthmessedup() || 
	$errorstring) {
	&Apache::lonnet::logthis("Trying to kill myself");
	$Apache::lontexconvert::messedup=1;
	if (ref($Apache::lonxml::request)) {
	    $Apache::lonxml::request->child_terminate();
	} else {
	    my $request;
	    eval { $request=Apache->request; };
	    if (!$request) {
		eval { $request=Apache2::RequestUtil->request; };
	    }
	    if ($request) {
		$request->child_terminate();
	    } else {
		&Apache::lonnet::logthis("Unable to find a request to do child_terminate on");
	    }
	}
    }
    return $xmlstring;
}

sub clean_out_math_mode {
    my ($texstring)=@_;
    $$texstring=~s/(?<!\\)\$//g;
    $$texstring=~s/\\[\)\(\]\[]//g;
    $$texstring=~s/\\ensuremath//g;
    return '';
}

sub displaystyle {
    my ($texstring)=@_;
    #has a $$ or \[ or \displaystyle in it, guessinng it's display mode
    if ($$texstring=~/[^\\]\$\$/ ||
	$$texstring=~/\\\[/ ||
	$$texstring=~/\\displaystyle/) { return 1; }
    return 0;
}

sub jsMath_converted {
    my $texstring=shift;
    my $tag='span';
    if (&displaystyle($texstring)) { $tag='div'; }
    &clean_out_math_mode($texstring);
    return &jsMath_header().
	'<'.$tag.' class="math">'.$$texstring.'</'.$tag.'>';
}

{
    my @jsMath_sent_header;
    sub jsMath_reset {
	undef(@jsMath_sent_header);
    }
    sub jsMath_push {
	push(@jsMath_sent_header,0);
    }
    sub jsMath_header {
	if (!@jsMath_sent_header) {
	    &Apache::lonnet::logthis("mismatched calls of jsMath_header and jsMath_process");
	    return '';
	}
	return '' if $jsMath_sent_header[-1];
	$jsMath_sent_header[-1]=1;
	return
            '<script type="text/javascript">
                     function NoFontMessage () {}
                     jsMath = {Parser: {prototype: {environments: {\'eqnarray*\' :[\'Array\',null,null,\'rcl\',[5/18,5/18],3,\'D\']}}}};
                   </script>'."\n".
	    '<script type="text/javascript" src="/adm/jsMath/jsMath.js"></script>'."\n";
    }
    sub jsMath_process {
	my $state = pop(@jsMath_sent_header);
	return '' if !$state;
	return "\n".
	    '<script type="text/javascript">jsMath.Process()</script>'."\n";
    }
    sub jsMath_state {
	my ($level) = @_;
	return $jsMath_sent_header[$level];
    }
}

sub tex_engine {
    if (exists($env{'form.texengine'})) {
	if ($env{'form.texengine'} ne '') {
            return $env{'form.texengine'};
        }
    }    
    if ($env{'request.course.id'}
	&& exists($env{'course.'.$env{'request.course.id'}.'.texengine'})) {
	return $env{'course.'.$env{'request.course.id'}.'.texengine'};
    }
    if (exists($env{'environment.texengine'})) {
	return $env{'environment.texengine'};
    }
    return 'tth';
}

sub init_math_support {
    my ($inherit_jsmath) = @_;
    &init_tth();
    &Apache::lontexconvert::jsMath_push();
    if (lc(&tex_engine()) eq 'jsmath' ||
	($inherit_jsmath && &jsMath_state(-2))) {
	return &Apache::lontexconvert::jsMath_header();
    }
    return;
}

sub mimetex_converted {
    my $texstring=shift;
    my $displaystyle=&displaystyle($texstring);

    &clean_out_math_mode($texstring);

    if ($displaystyle) {
	$$texstring='\\displaystyle \\Large '.$$texstring;
    }
    my $result='<img src="/cgi-bin/mimetex.cgi?'.&escape($$texstring).'" />';
    if ($displaystyle) {
	$result='<center>'.$result.'</center>';
    }
    return $result;
}

sub converted {
    my ($string,$mode)=@_;
    if ($mode eq '') { $mode = &tex_engine(); }
    if ($mode =~ /tth/i) {
	return &tth_converted($string);
    } elsif ($mode =~ /jsmath/i) {
	return &jsMath_converted($string);
    } elsif ($mode =~ /mimetex/i) {
	return &mimetex_converted($string);
    }
    return &tth_converted($string);
}

# ------------------------------------------------------------ Message display

sub to_convert {
    my ($string) = @_;
    $string=~s/\<br\s*\/?\>/ /gs;
#    $string=~s/\s/ /gs;
    $string=&HTML::Entities::decode($string);
    return &converted(\$string);
}

sub smiley {
    my $expression=shift;
    if ($env{'browser.imagesuppress'} eq 'on') { return $expression; }
    my %smileys=('\:\-\)' => 'smiley',
		 '8\-\)'  => 'coolsmile',
		 '8\-(I|\|)'   => 'coolindiff',
		 ':\-(I|\|)'   => 'neutral',
		 '\:\-(o|O|\(\))' => 'shocked',
		 ':\-\('  => 'frowny',
		 '\;\-\)' => 'wink',
		 '\:\-P'  => 'baeh',
		 '\:\-(\\\|\\/)' => 'hrrm',
		 '\:\-D'  => 'bigsmile',
		 '\:\-C'  => 'angry',
		 '\:(\'|\`)\-\(' => 'cry',
		 '\:\-(X|\#)' => 'lipsrsealed',
		 '\:\-S' => 'huh');
    my $iconpath=$Apache::lonnet::perlvar{'lonIconsURL'};
    foreach my $smiley (keys(%smileys)) {
	$expression=~s/$smiley/\<img src="$iconpath\/$smileys{$smiley}.gif" \/\>/gs; 
    }
    return $expression;
}

sub msgtexconverted {
    my ($message,$email) = @_;
    $errorstring='';
    &init_tth();
    my $outmessage='';
    my $tex=0;
    foreach my $fragment (split(/(?:\&lt\;|\<)\/*m\s*(?:\&gt\;|\>)/i,$message)) {
	if ($tex) {
	    if ($email) {
		$outmessage.='</pre><tt>'.&to_convert($fragment).'</tt><pre>';
		$tex=0;
	    } else {
		$outmessage.=&to_convert($fragment);
		$tex=0;
	    }
	} else {
            $outmessage.=&smiley($fragment);
	    $tex=1;
	}
    }
    $message=$outmessage; $outmessage=''; $tex=0;
    foreach my $fragment (split(/(?:\&lt\;|\<)\/*algebra\s*(?:\&gt\;|\>)/i,
				$message)) {
	if ($tex) {
	    if ($email) {
		$outmessage.='</pre><tt>'.&algebra($fragment,'web').'</tt><pre>';
		$tex=0;
	    } else {
		$outmessage.=&algebra($fragment,'web');
		$tex=0;
	    }
	} else {
            $outmessage.=$fragment;
	    $tex=1;
	}
    }
    if (wantarray) {
	return ($outmessage,$errorstring);
    } else {
	return $outmessage.$errorstring;
    }
}

sub algebra {
    use AlgParser;

    my ($string,$target,$style,$parstack,$safeeval)=@_;
    my $parser = new AlgParserWithImplicitExpand;
    $string=&prepare_algebra($string);
    my $ret = $parser->parse($string);
    my $result='['.&mt('Algebra unconverted due to previous errors').']';
    if ( ref($ret) ) {
	#$parser->tostring();
	$parser->normalize();
	my $latex=$parser->tolatex();
	$latex=&postprocess_algebra($latex);
	if ($style eq 'display') {
	    $latex='$$'.$latex.'$$x';
	} else {
	    # style is 'inline'
	    $latex='\\ensuremath{'.$latex.'}';
	}
	if ($target eq 'web' || $target eq 'analyze') {
            my $display=&Apache::lonxml::get_param('display',$parstack,$safeeval);
            $result = &converted(\$latex,$display);
#	    $result = &converted(\$latex);
	} else {
	    $result = $latex;
	}
    } else {
	&Apache::lonxml::error($parser->{'htmlerror'});
    }
}

sub prepare_algebra {
    my ($string)=@_;

    # makes the decision about what is a minus sign easier supposedly
    $string =~ s/(\<\>|\<\=|\>\=[\=\>\<] *)-/$1 zeroplace -/g;

    return $string;
}

sub postprocess_algebra {
    my ($string)=@_;
    
    # moodle had these and I don't know why, ignoring them for now
    # $string =~s/\\fun/ /g;

    # sqrt(3,4) means the 4 root of 3
    $string =~s/\\sqrt{([^,]+),([^\}]+)}/\\sqrt[$2]{$1}/gs;

    # log(3,4) means the log base 4 of 3
    $string =~s/\\log\\left\((.+?),(.+?)\\right\)/\\log_{$2}\\left($1\\right)/gs;

    # log(3,4) means the log base 4 of 3
    $string =~s/\\((?:sin|cos|tan|sec|csc|cot)(?:h)?)\\left\((.+?),(.+?)\\right\)/\\$1^{$3}\\left($2\\right)/gs;

    # int(3,a,b) integral from a to b of 3
    $string =~s/\\int\\left\((.+?),(.+?),(.+?)\\right\)/\\int_{$2}^{$3}\\left($1\\right)/gs;

    # int( ... dx) -> ...
    $string =~s/\\int\\left\((.+?)d[a-z]\\right\)/$1/gs;

    # 
    $string =~s/\\lim\\left\((.+?),(.+?),(.+?)\\right\)/\\lim_{$2\\to $3}$1/gs;
    return $string;
}
1;
__END__

==> loncapa-2.7.99.0.tar.gz: loncapa-2.7.99.0/loncom/xml/lontexconvert.pm <==
# The LearningOnline Network with CAPA
# TeX Conversion Module
#
# $Id: lontexconvert.pm,v 1.90 2008/11/18 10:02:56 hauer Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# Copyright for TtHfunc and TtMfunc by Ian Hutchinson. 
# TtHfunc and TtMfunc (the "Code") may be compiled and linked into 
# binary executable programs or libraries distributed by the 
# Michigan State University (the "Licensee"), but any binaries so 
# distributed are hereby licensed only for use in the context
# of a program or computational system for which the Licensee is the 
# primary author or distributor, and which performs substantial 
# additional tasks beyond the translation of (La)TeX into HTML.
# The C source of the Code may not be distributed by the Licensee
# to any other parties under any circumstances.
#

package Apache::lontexconvert;

use strict;
use tth();
use vars qw($errorstring);
#use Apache::lonxml();
use Apache::lonlocal;
use Apache::lonnet;
use lib '/home/httpd/lib/perl/';
use LONCAPA;
 

# ====================================================================== Header

sub init_tth {
    my $options=$env{'course.'.$env{'request.course.id'}.'.tthoptions'};
    if ($options =~ /\S/) {
	$options = ' '.$options;
    } else {
	undef($options);
    }
    if ($env{'browser.mathml'}) {
	&tth::ttminit();
	if ($env{'browser.unicode'}) {
	    &tth::ttmoptions('-L -u1'.$options);
	} else {
	    &tth::ttmoptions('-L -u0'.$options);
	}
    } else {
	&tth::tthinit();
	if ($env{'browser.unicode'}) {
	    &tth::tthoptions('-L -u1'.$options);
	} else {
	    &tth::tthoptions('-L -u0'.$options);
	}
    }
}

# ================================================================== Conversion

$Apache::lontexconvert::messedup=0;

# we need this routine because &converted can get called from inside
# of the safespace (through &xmlparse('<m>stuff</m>') which doesn't
# allow the opcode for alarm, so we need to compile this before we get
# into the safe space since opcode checks only occur at compile time
sub convert_real {
    my ($texstring)=@_;
    my ($xmlstring,$errorstring);
    local $SIG{SEGV}=sub { $Apache::lontexconvert::messedup=1; die; };
    local $SIG{ALRM}=sub { 
	&Apache::lonnet::logthis("ALRM");
	$xmlstring='['.&mt("TeX unconverted due to errors").']';
	$Apache::lontexconvert::messedup=1;
	die &mt("TeX unconverted due to errors");
    };
    &Apache::lonxml::start_alarm();
    if ($env{'browser.mathml'}) {
	$xmlstring=&tth::ttm($$texstring);
	$xmlstring=~s/\<math\>/\<math xmlns=\"\&mathns\;\"\>/g;
	$xmlstring=~s/\<br\>/\<br\/\>/g;
	$xmlstring=~s/\<p\>/\<p\>\<\/p\>/g;
	$errorstring.=&tth::ttmerror();
    } else {
	$xmlstring=&tth::tth($$texstring);
	$errorstring.=&tth::ttherror();
	$xmlstring=~s-</font(\s*)>-</font>-g;
    }
    $xmlstring=~s/^\s*\<br clear\=\"all\"/\<br/s;
    $xmlstring=~s/^\s*//;
    $xmlstring=~s/\s*$//;
    #
    # \rightleftharpoons is not converted by tth but maps
    # reasonably well to &#8660;.  If we get many more of these,
    # we're going to need to have a translation sub.
    #
    my $lrharpoon = pack("U", 0x21cc);
    $xmlstring=~s/\\rightleftharpoons/$lrharpoon/g;

    &Apache::lonxml::end_alarm();
    return ($xmlstring,$errorstring);
}

sub tth_converted {
    my $texstring=shift;
    my $xmlstring='['.&mt('UNDISPLAYABLE').']';
    if ($Apache::lontexconvert::messedup) {
	return '['.&mt('TeX unconverted due to previous errors').']';
    }
    $$texstring ='\\documentstyle{article}'.$$texstring;

    eval(<<'ENDCONV');
    ($xmlstring,$errorstring)=&convert_real($texstring)
ENDCONV
    if ($@) {
	$errorstring.=&mt("Evaluation Error: ").$@;
	$Apache::lontexconvert::messedup=1;
    }
    if ($Apache::lontexconvert::messedup || &tth::tthmessedup() || 
	$errorstring) {
	&Apache::lonnet::logthis("Trying to kill myself");
	$Apache::lontexconvert::messedup=1;
	if (ref($Apache::lonxml::request)) {
	    $Apache::lonxml::request->child_terminate();
	} else {
	    my $request;
	    eval { $request=Apache->request; };
	    if (!$request) {
		eval { $request=Apache2::RequestUtil->request; };
	    }
	    if ($request) {
		$request->child_terminate();
	    } else {
		&Apache::lonnet::logthis("Unable to find a request to do child_terminate on");
	    }
	}
    }
    return $xmlstring;
}

sub clean_out_math_mode {
    my ($texstring)=@_;
    $$texstring=~s/(?<!\\)\$//g;
    $$texstring=~s/\\[\)\(\]\[]//g;
    $$texstring=~s/\\ensuremath//g;
    return '';
}

sub displaystyle {
    my ($texstring)=@_;
    #has a $$ or \[ or \displaystyle in it, guessinng it's display mode
    if ($$texstring=~/[^\\]\$\$/ ||
	$$texstring=~/\\\[/ ||
	$$texstring=~/\\displaystyle/) { return 1; }
    return 0;
}

sub jsMath_converted {
    my $texstring=shift;
    my $tag='span';
    if (&displaystyle($texstring)) { $tag='div'; }
    &clean_out_math_mode($texstring);
    return &jsMath_header().
	'<'.$tag.' class="math">'.$$texstring.'</'.$tag.'>';
}

{
    my @jsMath_sent_header;
    sub jsMath_reset {
	undef(@jsMath_sent_header);
    }
    sub jsMath_push {
	push(@jsMath_sent_header,0);
    }
    sub jsMath_header {
	if (!@jsMath_sent_header) {
	    &Apache::lonnet::logthis("mismatched calls of jsMath_header and jsMath_process");
	    return '';
	}
	return '' if $jsMath_sent_header[-1];
	$jsMath_sent_header[-1]=1;
	return
            '<script type="text/javascript">
                     function NoFontMessage () {}
                     jsMath = {Parser: {prototype: {environments: {\'eqnarray*\' :[\'Array\',null,null,\'rcl\',[5/18,5/18],3,\'D\']}}}};
                   </script>'."\n".
	    '<script type="text/javascript" src="/adm/jsMath/jsMath.js"></script>'."\n";
    }
    sub jsMath_process {
	my $state = pop(@jsMath_sent_header);
	return '' if !$state;
	return "\n".
	    '<script type="text/javascript">jsMath.Process()</script>'."\n";
    }
    sub jsMath_state {
	my ($level) = @_;
	return $jsMath_sent_header[$level];
    }
}

sub tex_engine {
    if (exists($env{'form.texengine'})) {
	if ($env{'form.texengine'} ne '') {
            return $env{'form.texengine'};
        }
    }    
    if ($env{'request.course.id'}
	&& exists($env{'course.'.$env{'request.course.id'}.'.texengine'})) {
	return $env{'course.'.$env{'request.course.id'}.'.texengine'};
    }
    if (exists($env{'environment.texengine'})) {
	return $env{'environment.texengine'};
    }
    return 'tth';
}

sub init_math_support {
    my ($inherit_jsmath) = @_;
    &init_tth();
    &Apache::lontexconvert::jsMath_push();
    if (lc(&tex_engine()) eq 'jsmath' ||
	($inherit_jsmath && &jsMath_state(-2))) {
	return &Apache::lontexconvert::jsMath_header();
    }
    return;
}

sub mimetex_converted {
    my $texstring=shift;
    my $displaystyle=&displaystyle($texstring);

    &clean_out_math_mode($texstring);

    if ($displaystyle) {
	$$texstring='\\displaystyle \\Large '.$$texstring;
    }
    my $result='<img src="/cgi-bin/mimetex.cgi?'.&escape($$texstring).'" alt="$'.$$texstring.'$" />';
    if ($displaystyle) {
	$result='<center>'.$result.'</center>';
    }
    return $result;
}

sub converted {
    my ($string,$mode)=@_;
    if ($mode eq '') { $mode = &tex_engine(); }
    if ($mode =~ /tth/i) {
	return &tth_converted($string);
    } elsif ($mode =~ /jsmath/i) {
	return &jsMath_converted($string);
    } elsif ($mode =~ /mimetex/i) {
	return &mimetex_converted($string);
    }
    return &tth_converted($string);
}

# ------------------------------------------------------------ Message display

sub to_convert {
    my ($string) = @_;
    $string=~s/\<br\s*\/?\>/ /gs;
#    $string=~s/\s/ /gs;
    $string=&HTML::Entities::decode($string);
    return &converted(\$string);
}

sub smiley {
    my $expression=shift;
    if ($env{'browser.imagesuppress'} eq 'on') { return $expression; }
    my %smileys=('\:\-\)' => 'smiley',
		 '8\-\)'  => 'coolsmile',
		 '8\-(I|\|)'   => 'coolindiff',
		 ':\-(I|\|)'   => 'neutral',
		 '\:\-(o|O|\(\))' => 'shocked',
		 ':\-\('  => 'frowny',
		 '\;\-\)' => 'wink',
		 '\:\-P'  => 'baeh',
		 '\:\-(\\\|\\/)' => 'hrrm',
		 '\:\-D'  => 'bigsmile',
		 '\:\-C'  => 'angry',
		 '\:(\'|\`)\-\(' => 'cry',
		 '\:\-(X|\#)' => 'lipsrsealed',
		 '\:\-S' => 'huh');
    my $iconpath=$Apache::lonnet::perlvar{'lonIconsURL'};
    foreach my $smiley (keys(%smileys)) {
	$expression=~s/$smiley/\<img src="$iconpath\/$smileys{$smiley}.gif" \/\>/gs; 
    }
    return $expression;
}

sub msgtexconverted {
    my ($message,$email) = @_;
    $errorstring='';
    &init_tth();
    my $outmessage='';
    my $tex=0;
    foreach my $fragment (split(/(?:\&lt\;|\<)\/*m\s*(?:\&gt\;|\>)/i,$message)) {
	if ($tex) {
	    if ($email) {
		$outmessage.='</pre><tt>'.&to_convert($fragment).'</tt><pre>';
		$tex=0;
	    } else {
		$outmessage.=&to_convert($fragment);
		$tex=0;
	    }
	} else {
            $outmessage.=&smiley($fragment);
	    $tex=1;
	}
    }
    $message=$outmessage; $outmessage=''; $tex=0;
    foreach my $fragment (split(/(?:\&lt\;|\<)\/*algebra\s*(?:\&gt\;|\>)/i,
				$message)) {
	if ($tex) {
	    if ($email) {
		$outmessage.='</pre><tt>'.&algebra($fragment,'web').'</tt><pre>';
		$tex=0;
	    } else {
		$outmessage.=&algebra($fragment,'web');
		$tex=0;
	    }
	} else {
            $outmessage.=$fragment;
	    $tex=1;
	}
    }
    if (wantarray) {
	return ($outmessage,$errorstring);
    } else {
	return $outmessage.$errorstring;
    }
}

sub algebra {
    use AlgParser;

    my ($string,$target,$style,$parstack,$safeeval)=@_;
    my $parser = new AlgParserWithImplicitExpand;
    $string=&prepare_algebra($string);
    my $ret = $parser->parse($string);
    my $result='['.&mt('Algebra unconverted due to previous errors').']';
    if ( ref($ret) ) {
	#$parser->tostring();
	$parser->normalize();
	my $latex=$parser->tolatex();
	$latex=&postprocess_algebra($latex);
	if ($style eq 'display') {
	    $latex='$$'.$latex.'$$x';
	} else {
	    # style is 'inline'
	    $latex='\\ensuremath{'.$latex.'}';
	}
	if ($target eq 'web' || $target eq 'analyze') {
            my $display=&Apache::lonxml::get_param('display',$parstack,$safeeval);
            $result = &converted(\$latex,$display);
#	    $result = &converted(\$latex);
	} else {
	    $result = $latex;
	}
    } else {
	&Apache::lonxml::error($parser->{'htmlerror'});
    }
}

sub prepare_algebra {
    my ($string)=@_;

    # makes the decision about what is a minus sign easier supposedly
    $string =~ s/(\<\>|\<\=|\>\=[\=\>\<] *)-/$1 zeroplace -/g;

    return $string;
}

sub postprocess_algebra {
    my ($string)=@_;
    
    # moodle had these and I don't know why, ignoring them for now
    # $string =~s/\\fun/ /g;

    # sqrt(3,4) means the 4 root of 3
    $string =~s/\\sqrt{([^,]+),([^\}]+)}/\\sqrt[$2]{$1}/gs;

    # log(3,4) means the log base 4 of 3
    $string =~s/\\log\\left\((.+?),(.+?)\\right\)/\\log_{$2}\\left($1\\right)/gs;

    # log(3,4) means the log base 4 of 3
    $string =~s/\\((?:sin|cos|tan|sec|csc|cot)(?:h)?)\\left\((.+?),(.+?)\\right\)/\\$1^{$3}\\left($2\\right)/gs;

    # int(3,a,b) integral from a to b of 3
    $string =~s/\\int\\left\((.+?),(.+?),(.+?)\\right\)/\\int_{$2}^{$3}\\left($1\\right)/gs;

    # int( ... dx) -> ...
    $string =~s/\\int\\left\((.+?)d[a-z]\\right\)/$1/gs;

    # 
    $string =~s/\\lim\\left\((.+?),(.+?),(.+?)\\right\)/\\lim_{$2\\to $3}$1/gs;
    return $string;
}
1;
__END__
